using System.Text.Json;
using MediatR;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.SignalR;
using Todo.Api.Hubs.Models;
using Todo.Core.Application.Commands.Todo;
using Todo.Core.Application.Services.UserConnectionStore;
using Todo.Core.Interfaces.Persistence;
using Todo.Core.Interfaces.User;

namespace Todo.Api.Hubs;

[Authorize]
public class TodoHub : Hub
{
    private readonly ICurrentUserService _currentUserService;
    private readonly IUserConnectionStore _userConnectionStore;

    public TodoHub(
        IUserConnectionStore userConnectionStore,
        ICurrentUserService currentUserService)
    {
        _userConnectionStore = userConnectionStore;
        _currentUserService = currentUserService;
    }

    public override Task OnConnectedAsync()
    {
        var userId = _currentUserService.GetUserId();
        if (userId is not null)
            _userConnectionStore.AddAsync(userId, Context.ConnectionId);
        return base.OnConnectedAsync();
    }

    public override Task OnDisconnectedAsync(Exception? exception)
    {
        var userId = _currentUserService.GetUserId();
        if (userId is not null)
            _userConnectionStore.RemoveAsync(userId, Context.ConnectionId);

        return base.OnDisconnectedAsync(exception);
    }
}